'use strict';

var fs = require('fs');
var path = require('path');
var matched = require('matched');
var htmlImportsVisitor = require('html-imports-visitor');
var parse5 = require('parse5');
var dom5 = require('dom5');

const mkdir = (path$$1) => {
  try {
    fs.readdirSync(path$$1);
  } catch (err) {
    mkdir(path.dirname(path$$1));
    fs.mkdirSync(path$$1);
  }
};

const writeFileEx = (path$$1, doc) => (res, rej) => {
  fs.writeFile(path$$1, parse5.serialize(doc), (err) => err ? rej(err) : res());
};

const mkdirEx = (path$$1) => (res, rej) => {
  try {
    mkdir(path$$1);
    res();
  } catch (err) {
    rej(err);
  }
};

const writeHtmls = (htmls, destPath) => {

  const executors = {};
  for (const filepath in htmls) {
    const destFilepath = path.resolve(destPath, path.relative(process.cwd(), filepath));
    const destDirpath = path.dirname(destFilepath);
    const ex = writeFileEx(destFilepath, htmls[filepath]);
    if (executors[destDirpath]) {
      executors[destDirpath].push(ex);
    } else {
      executors[destDirpath] = [ex];
    }
  }

  return Promise.all(
    Object.keys(executors).map(
      (path$$1) => new Promise(mkdirEx(path$$1)).then(
        Promise.all(
          executors[path$$1].map((ex) => new Promise(ex))
        )
      )
    )
  )
};

const entry = "\0rollup-plugin-html-entry:entry-point";

var index = (config) => {
  let include = "**/*.html";
  let exclude = [];
  let external = [];
  let exporter = (path$$1) => `export * from ${JSON.stringify(path$$1)};`;
  let output = false;

  const configure = (options) => {
    if (typeof options === "string") {
      include = [options];
    } else if (Array.isArray(options)) {
      include = options;
    } else {
      include = options.include || [];
      exclude = options.exclude || [];
      external = options.external || [];
      if (options.exports === false) {
        exporter = (path$$1) => `import ${JSON.stringify(path$$1)};`;
      }
    }

    if (options.output) {
      output = options.output;
    }
  };

  if (config) {
    configure(config);
  }

  let realPaths;
  let virtualPaths;
  let pathsList;
  let htmls;
  let excludeMap;

  const visitor = {

    import(link, ref) {
      var location = ref.location;

      if (excludeMap[location]) {
        // remove <link rel="import"> and avoid entering the doc
        dom5.remove(link);
      }
    },

    enter(document, ref) {
      var path$$1 = ref.path;

      htmls[path$$1] = document;
    },

    visit(element, ref) {
      var path$$1 = ref.path;
      var index = ref.index;

      realPaths[path$$1] = true;
      const src = dom5.getAttribute(element, "src");
      if (src) {
        if (htmlImportsVisitor.isLocal(src)) {
          pathsList.push(htmlImportsVisitor.resolve(path$$1, src));
          if (output) {
            dom5.remove(element);
          }
        }
      } else {
        // (todo) should avoid possibile collision against real paths
        const virtualPath = `${path$$1}_${index}.js`;
        pathsList.push(virtualPath);
        virtualPaths[virtualPath] = dom5.getTextContent(element);
        if (output) {
          dom5.remove(element);
        }
      }
    }
  };

  const analyze = () => {

    // Reset
    realPaths = {};
    virtualPaths = {};
    pathsList = [];
    htmls = {};
    excludeMap = {};

    const helper = new htmlImportsVisitor.VisitorHelper(visitor, htmlImportsVisitor.predicates.hasTagName("script"));

    if (exclude.length) {
      const excluded = matched.sync(exclude, { realpath: true });
      for (const i in excluded) {
        excludeMap[excluded[i]] = true;
      }
    }

    if (external.length) {
      const omitted = matched.sync(
        external,
        { realpath: true }
      );
      for (const i in omitted) {
        helper.omit(omitted[i]);
      }
    }

    if (include.length) {
      const included = matched.sync(
        include.concat(exclude.map((pattern) => `!${pattern}`))
               .concat(external.map((pattern) => `!${pattern}`)),
        { realpath: true }
      );
      for (const i in included) {
        helper.enter(included[i]);
      }
    }
  };

  return {
    options(options) {
      if (options.input && options.input !== entry) {
        configure(options.input);
      }
      options.input = entry;
      analyze();
    },

    resolveId(id) {
      if (id === entry) {
        return entry
      }

      if (id in virtualPaths) {
        return id
      }
    },

    load(id) {
      if (id === entry) {
        if (!pathsList.length) {
          return Promise.resolve("")
        }
        return new Promise(
          (res) => res(pathsList.map(exporter).join("\n"))
        )
      }

      if (id in virtualPaths) {
        return virtualPaths[id]
      }
    },

    ongenerate() {
      if (output) {
        return writeHtmls(htmls, output)
      }
    }
  }

};

module.exports = index;
